"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validate = exports.lookup = exports.referenceCheck = void 0;
const jsonpath_1 = require("jsonpath");
/**
 * Sorting mechanism for [[IQueryResult]]
 *
 * @param a - first element
 * @param b - second element
 * @return -1 for a less than b, 0 for a equals b, 1 for a greater than b
 *
 * Example:
 * ``` js
 * querySorter({"path": [], "value": "a"}, {"path": [], "value": "b"}); //=> -1
 * querySorter({"path": [], "value": true}, {"path": [], "value": true}); //=> 0
 * querySorter({"path": [], "value": 2}, {"path": [], "value": 1}); //=> 1
 * ```
 */
function querySorter(a, b) {
    if (a.value < b.value) {
        return -1;
    }
    else if (a.value > b.value) {
        return 1;
    }
    else {
        return 0;
    }
}
/**
 * Checks that all keyrefs have an associated key
 *
 * @param keyrefs - references to a key
 * @param keys - identifier keys
 * @return validity status and any errors found
 *
 * Example:
 * ``` js
 * var keyrefs = [{"path": ["$", "some", "path"], "value": "example"}];
 * var keys = [{"path": ["$", "some", "other", "path"], "value": "example"}];
 *
 * var result = referenceCheck(keyrefs, keys);
 *
 * console.log(result); //=> {"isValid": true, "errors": []}
 * ```
 */
function referenceCheck(keyrefs, keys) {
    keyrefs.sort(querySorter);
    keys.sort(querySorter);
    let isValid = true;
    const errors = [];
    let keyIndex = 0;
    let keyrefIndex = 0;
    while (keyrefIndex < keyrefs.length) {
        const key = keys[keyIndex];
        const reference = keyrefs[keyrefIndex];
        if (keyIndex >= keys.length || reference.value < key.value) {
            keyrefIndex++;
            isValid = false;
            errors.push(reference);
        }
        else if (reference.value === key.value) {
            keyrefIndex++;
        }
        else {
            keyIndex++;
        }
    }
    return {
        errors,
        isValid,
    };
}
exports.referenceCheck = referenceCheck;
/**
 * Searches for a set of jsonpath queries.
 *
 * @param queries - identifier and jsonpath pairings to look up
 * @param document - JSON document that will be searched
 * @return identifier and results pairings found in the document
 *
 * Example:
 * ``` js
 * var queries = [{"one": "$.a"}, {"two": "$.b"}];
 * var document = {"a": "something", "b": "something else"};
 *
 * var result = lookup(queries, document);
 *
 * console.log(result);
 * // {
 * //   "one": [{"path": ["$", "a"], "value": "something"}],
 * //   "two": [{"path": ["$", "b"], "value": "something else"}]
 * // }
 * ```
 */
function lookup(queries, document) {
    const results = {};
    Object.keys(queries).forEach((identifier) => (results[identifier] = jsonpath_1.nodes(document, queries[identifier])));
    return results;
}
exports.lookup = lookup;
/**
 * Validates a JSON document against a JSON schema with keyref and key constraints defined
 *
 * @param document - document to be validated
 * @param schema - schema with keyref and key groups defined
 *
 * Example:
 * ``` js
 * var document = {"a": "value", "b": "value"};
 * var schema = {
 *   "keyrefs": {"identifier": "$.a"},
 *   "keys": {"identifier": "$.b"}
 * };
 *
 * var result = validate(document, schema);
 *
 * console.log(result); //=> {"errors": [], "isValid": true}
 * ```
 */
function validate(document, schema) {
    const keyrefs = lookup(schema.keyrefs, document);
    const keys = lookup(schema.keys, document);
    return (Object.keys(keyrefs)
        // Check keyrefs and keys for a single identifier group
        .map((identifier) => {
        // Ensure keyrefs identifier has a matching keys identifier
        if (!keys[identifier]) {
            return {
                errors: [{ path: ["$"], value: identifier }],
                isValid: false,
            };
        }
        // Check that keyrefs have matching keys
        return referenceCheck(keyrefs[identifier], keys[identifier]);
    })
        // Merge results from all identifiers together
        .reduce((overall, current) => {
        return {
            errors: overall.errors.concat(current.errors),
            isValid: overall.isValid && current.isValid,
        };
    }, {
        errors: [],
        isValid: true,
    }));
}
exports.validate = validate;
//# sourceMappingURL=index.js.map